@@ -18,6 +18,8 @@
 import itertools
 import sys
 
+from openstack import exceptions as sdk_exceptions
+from openstack.network.v2 import quota as _quota
 from osc_lib.command import command
 from osc_lib import utils
 import six
@@ -80,7 +82,7 @@ NETWORK_QUOTAS = {
 
 
 class SetQuota(command.Command):
-    """Set quotas for project or class"""
+    _description = _("Set quotas for project or class")
 
     def _build_options_list(self):
         if self.app.client_manager.is_network_endpoint_enabled():
@@ -186,7 +188,7 @@ class SetQuota(command.Command):
 
 
 class ShowQuota(command.ShowOne):
-    """Show quotas for project or class"""
+    _description = _("Show quotas for project or class")
 
     def get_parser(self, prog_name):
         parser = super(ShowQuota, self).get_parser(prog_name)
@@ -251,7 +253,39 @@ class ShowQuota(command.ShowOne):
             project = self._get_project(parsed_args)
             client = self.app.client_manager.network
             if parsed_args.default:
-                network_quota = client.get_quota_default(project)
+                # TODO(dtroyer): Remove the top of this if block once the
+                #                fixed SDK QuotaDefault class is the minimum
+                #                required version.  This is expected to be
+                #                SDK release 0.9.13
+                if hasattr(_quota.QuotaDefault, 'project'):
+                    # hack 0.9.11+
+                    quotadef_obj = client._get_resource(
+                        _quota.QuotaDefault,
+                        project,
+                    )
+                    quotadef_obj.base_path = quotadef_obj.base_path % {
+                        'project': project,
+                    }
+                    try:
+                        network_quota = quotadef_obj.get(
+                            client.session,
+                            requires_id=False,
+                        )
+                    except sdk_exceptions.NotFoundException as e:
+                        raise sdk_exceptions.ResourceNotFound(
+                            message="No %s found for %s" %
+                                    (_quota.QuotaDefault.__name__, project),
+                            details=e.details,
+                            response=e.response,
+                            request_id=e.request_id,
+                            url=e.url,
+                            method=e.method,
+                            http_status=e.http_status,
+                            cause=e.cause,
+                        )
+                    # end hack-around
+                else:
+                    network_quota = client.get_quota_default(project)
             else:
                 network_quota = client.get_quota(project)
             return network_quota
@@ -273,6 +307,10 @@ class ShowQuota(command.ShowOne):
         volume_quota_info = self.get_compute_volume_quota(volume_client,
                                                           parsed_args)
         network_quota_info = self.get_network_quota(parsed_args)
+        # NOTE(reedip): Remove the below check once requirement for
+        #               Openstack SDK is fixed to version 0.9.12 and above
+        if type(network_quota_info) is not dict:
+            network_quota_info = network_quota_info.to_dict()
 
         info = {}
         info.update(compute_quota_info)
