@@ -18,6 +18,7 @@ from osc_lib import exceptions
 from osc_lib import utils as osc_utils
 
 from openstackclient.network.v2 import router
+from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes_v3
 from openstackclient.tests.unit.network.v2 import fakes as network_fakes
 from openstackclient.tests.unit import utils as tests_utils
 
@@ -29,6 +30,7 @@ class TestRouter(network_fakes.TestNetworkV2):
 
         # Get a shortcut to the network client
         self.network = self.app.client_manager.network
+        self.projects_mock = self.app.client_manager.identity.projects
 
 
 class TestAddPortToRouter(TestRouter):
@@ -457,7 +459,7 @@ class TestListRouter(TestRouter):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.routers.assert_called_once_with(
-            **{'admin_state_up': True}
+            **{'admin_state_up': True, 'is_admin_state_up': True}
         )
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
@@ -474,8 +476,47 @@ class TestListRouter(TestRouter):
         columns, data = self.cmd.take_action(parsed_args)
 
         self.network.routers.assert_called_once_with(
-            **{'admin_state_up': False}
+            **{'admin_state_up': False, 'is_admin_state_up': False}
         )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_router_list_project(self):
+        project = identity_fakes_v3.FakeProject.create_one_project()
+        self.projects_mock.get.return_value = project
+        arglist = [
+            '--project', project.id,
+        ]
+        verifylist = [
+            ('project', project.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        filters = {'tenant_id': project.id, 'project_id': project.id}
+
+        self.network.routers.assert_called_once_with(**filters)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
+    def test_router_list_project_domain(self):
+        project = identity_fakes_v3.FakeProject.create_one_project()
+        self.projects_mock.get.return_value = project
+        arglist = [
+            '--project', project.id,
+            '--project-domain', project.domain_id,
+        ]
+        verifylist = [
+            ('project', project.id),
+            ('project_domain', project.domain_id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        filters = {'tenant_id': project.id, 'project_id': project.id}
+
+        self.network.routers.assert_called_once_with(**filters)
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
 
@@ -561,17 +602,19 @@ class TestSetRouter(TestRouter):
 
     # The router to set.
     _default_route = {'destination': '10.20.20.0/24', 'nexthop': '10.20.30.1'}
+    _network = network_fakes.FakeNetwork.create_one_network()
+    _subnet = network_fakes.FakeSubnet.create_one_subnet()
     _router = network_fakes.FakeRouter.create_one_router(
         attrs={'routes': [_default_route]}
     )
 
     def setUp(self):
         super(TestSetRouter, self).setUp()
-
+        self.network.router_add_gateway = mock.Mock()
         self.network.update_router = mock.Mock(return_value=None)
-
         self.network.find_router = mock.Mock(return_value=self._router)
-
+        self.network.find_network = mock.Mock(return_value=self._network)
+        self.network.find_subnet = mock.Mock(return_value=self._subnet)
         # Get the command object to test
         self.cmd = router.SetRouter(self.app, self.namespace)
 
@@ -661,10 +704,10 @@ class TestSetRouter(TestRouter):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.take_action(parsed_args)
-
+        routes = [{'destination': '10.20.30.0/24',
+                   'nexthop': '10.20.30.1'}]
         attrs = {
-            'routes': self._router.routes + [{'destination': '10.20.30.0/24',
-                                             'nexthop': '10.20.30.1'}],
+            'routes': routes + self._router.routes
         }
         self.network.update_router.assert_called_once_with(
             self._router, **attrs)
@@ -690,21 +733,31 @@ class TestSetRouter(TestRouter):
             self._router, **attrs)
         self.assertIsNone(result)
 
-    def test_set_route_no_route(self):
+    def test_set_route_overwrite_route(self):
+        _testrouter = network_fakes.FakeRouter.create_one_router(
+            {'routes': [{"destination": "10.0.0.2",
+                         "nexthop": "1.1.1.1"}]})
+        self.network.find_router = mock.Mock(return_value=_testrouter)
         arglist = [
-            self._router.name,
+            _testrouter.name,
             '--route', 'destination=10.20.30.0/24,gateway=10.20.30.1',
             '--no-route',
         ]
         verifylist = [
-            ('router', self._router.name),
+            ('router', _testrouter.name),
             ('routes', [{'destination': '10.20.30.0/24',
                          'gateway': '10.20.30.1'}]),
             ('no_route', True),
         ]
-
-        self.assertRaises(tests_utils.ParserException, self.check_parser,
-                          self.cmd, arglist, verifylist)
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        attrs = {
+            'routes': [{'destination': '10.20.30.0/24',
+                        'nexthop': '10.20.30.1'}]
+        }
+        self.network.update_router.assert_called_once_with(
+            _testrouter, **attrs)
+        self.assertIsNone(result)
 
     def test_set_clear_routes(self):
         arglist = [
@@ -726,21 +779,31 @@ class TestSetRouter(TestRouter):
             self._router, **attrs)
         self.assertIsNone(result)
 
-    def test_set_route_clear_routes(self):
+    def test_overwrite_route_clear_routes(self):
+        _testrouter = network_fakes.FakeRouter.create_one_router(
+            {'routes': [{"destination": "10.0.0.2",
+                         "nexthop": "1.1.1.1"}]})
+        self.network.find_router = mock.Mock(return_value=_testrouter)
         arglist = [
-            self._router.name,
+            _testrouter.name,
             '--route', 'destination=10.20.30.0/24,gateway=10.20.30.1',
             '--clear-routes',
         ]
         verifylist = [
-            ('router', self._router.name),
+            ('router', _testrouter.name),
             ('routes', [{'destination': '10.20.30.0/24',
                          'gateway': '10.20.30.1'}]),
             ('clear_routes', True),
         ]
-
-        self.assertRaises(tests_utils.ParserException, self.check_parser,
-                          self.cmd, arglist, verifylist)
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        attrs = {
+            'routes': [{'destination': '10.20.30.0/24',
+                        'nexthop': '10.20.30.1'}]
+        }
+        self.network.update_router.assert_called_once_with(
+            _testrouter, **attrs)
+        self.assertIsNone(result)
 
     def test_set_nothing(self):
         arglist = [
@@ -758,6 +821,110 @@ class TestSetRouter(TestRouter):
             self._router, **attrs)
         self.assertIsNone(result)
 
+    def test_wrong_gateway_params(self):
+        arglist = [
+            "--fixed-ip", "subnet='abc'",
+            self._router.id,
+        ]
+        verifylist = [
+            ('fixed_ip', [{'subnet': "'abc'"}]),
+            ('router', self._router.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action, parsed_args)
+
+    def test_set_gateway_network_only(self):
+        arglist = [
+            "--external-gateway", self._network.id,
+            self._router.id,
+        ]
+        verifylist = [
+            ('external_gateway', self._network.id),
+            ('router', self._router.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.network.update_router.assert_called_with(
+            self._router, **{'external_gateway_info': {
+                'network_id': self._network.id}})
+        self.assertIsNone(result)
+
+    def test_set_gateway_options_subnet_only(self):
+        arglist = [
+            "--external-gateway", self._network.id,
+            "--fixed-ip", "subnet='abc'",
+            self._router.id,
+            '--enable-snat',
+        ]
+        verifylist = [
+            ('router', self._router.id),
+            ('external_gateway', self._network.id),
+            ('fixed_ip', [{'subnet': "'abc'"}]),
+            ('enable_snat', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.network.update_router.assert_called_with(
+            self._router, **{'external_gateway_info': {
+                'network_id': self._network.id,
+                'external_fixed_ips': [{
+                    'subnet_id': self._subnet.id, }],
+                'enable_snat': True, }})
+        self.assertIsNone(result)
+
+    def test_set_gateway_option_ipaddress_only(self):
+        arglist = [
+            "--external-gateway", self._network.id,
+            "--fixed-ip", "ip-address=10.0.1.1",
+            self._router.id,
+            '--enable-snat',
+        ]
+        verifylist = [
+            ('router', self._router.id),
+            ('external_gateway', self._network.id),
+            ('fixed_ip', [{'ip-address': "10.0.1.1"}]),
+            ('enable_snat', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.network.update_router.assert_called_with(
+            self._router, **{'external_gateway_info': {
+                'network_id': self._network.id,
+                'external_fixed_ips': [{
+                    'ip_address': "10.0.1.1", }],
+                'enable_snat': True, }})
+        self.assertIsNone(result)
+
+    def test_set_gateway_options_subnet_ipaddress(self):
+        arglist = [
+            "--external-gateway", self._network.id,
+            "--fixed-ip", "subnet='abc',ip-address=10.0.1.1",
+            self._router.id,
+            '--enable-snat',
+        ]
+        verifylist = [
+            ('router', self._router.id),
+            ('external_gateway', self._network.id),
+            ('fixed_ip', [{'subnet': "'abc'",
+                           'ip-address': "10.0.1.1"}]),
+            ('enable_snat', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.network.update_router.assert_called_with(
+            self._router, **{'external_gateway_info': {
+                'network_id': self._network.id,
+                'external_fixed_ips': [{
+                    'subnet_id': self._subnet.id,
+                    'ip_address': "10.0.1.1", }],
+                'enable_snat': True, }})
+        self.assertIsNone(result)
+
 
 class TestShowRouter(TestRouter):
 
@@ -874,3 +1041,16 @@ class TestUnsetRouter(TestRouter):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         self.assertRaises(exceptions.CommandError,
                           self.cmd.take_action, parsed_args)
+
+    def test_unset_router_external_gateway(self):
+        arglist = [
+            '--external-gateway',
+            self._testrouter.name,
+        ]
+        verifylist = [('external_gateway', True)]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        attrs = {'external_gateway_info': {}}
+        self.network.update_router.assert_called_once_with(
+            self._testrouter, **attrs)
+        self.assertIsNone(result)
